   LIST P=16F628A
   ERRORLEVEL      -302    ;SUPPRESS BANK SEL

;
; 03/03/04 First version of program
; 24/03/04 Changed crystal from 8 to 16MHz
; 27/03/04 added sonic virtual temperature
; 30/03/04 added calibration feature in HEX
; 18/04/04 added distance and temperature calibration in decimal
; 20/04/04 average wind measurements and dist /2 in cal
; 27/04/04 reduce transmit cycles (reduce ringing time)
; 05/05/04 changed DIR printout format, added pulse, delay1, delay2, toggle
;          display format
; 07/05/04 In CAL do measurements first then do offsets
; 11/05/04 each direction has its own offset, delay3 between outputs
; 31/10/04 removed delay3 and toggle(T) function added L, W and w
; 07/11/04 changed comms speed to 4800 baud, added delay to RDY_TX
; 21/11/04 compare X & Y speed of sound to validate data, added speed of sound
;          when finished calibration, changed 'w' to start measurement then
;          download data not the other way round, changed 'V' distance to
;          decimal
; 28/11/04 temperature input can be a negative number, added sonic virtual
;          temperature to w
; 07/01/05 zero timer1 then apply offsets after measurements if no timeout,
;          changed TXSTA,1 to PIR1,4, added write error to EEPROM, moved test
;          point 1 (port B,5) to port A,3. Removed test point 2 (port B,0)
;          both causing USART to lockup on PIC16F628A. If timeout set counter
;          to FFFF, added counters and diff between counters in adj trimpot
;          routine
; 19/08/07 added check for zero on DIV_BIN, check if result zero on DIV_70
; 08/09/09 removed dist WE
; 11/10/09 variable conflict in CAL4 routine
;          variable conflict in TIM_32 routine
;          added calculated counter
;
;          C - Write calibration bytes to internal EEPROM
;            - Enter Dist 0.xxxx in metres
;            - Enter Pulse xx, 03=3 cycles of 40kHz
;            - Enter Delay1 xx, D0=0.47025ms, Range 01=4.5us, 00=0.57825ms
;            - Enter Delay2 xx, 05=0.963ms, Range 01=0.195ms, 00=49.155ms
;            - Enter Temp xx.xx in degrees Celsius   ???.?? m/s
;
;          V - Read calibration bytes and display
;              distance (decimal), S offset, W offset, N offset, E offset,
;              number of Tx cycles, delay mask, delay
;              between measurements (Hex)
;
;          L - Loop and do function W
;
;          W - Output RS232 (bits_byte, 4 x time of flight, x, y, r, wind dir,
;              wind speed, sonic virtual temperature and speed of sound)
;              direct to PC in HEX and DEC.
;
;          w - Output RS232 (bits_byte, wind dir & wind speed, virtual
;              temperature & crc) direct to PC in HEX and DEC.
;
;          Terminal setup 4800,8,N,1
;
;          This program is experimential at this stage.
;
; RA0 17 to 4052-A1 (4 ch analog multiplexer/demultiplexer)
; RA1 18 to 4052-A0 (4 ch analog multiplexer/demultiplexer)
; RA2  1 40kHz out
; RA3  2 TEST POINT 1, for CRO to indicate start of measurement cycle (TRIG INPUT)
; RA4  3 to 4052 pin 13
;
; RB0  6
; RB1  7 RX (RS232) used for calibration process
; RB2  8 TX (RS232)
; RB3  9 CCP1 from ultrasonic transducer amplifier output
; RB4 10 PGM function - low voltage programing enabled
; RB5 11
; RB6 12 CLOCK (for programing)
; RB7 13 DATA (for programing)
;
; MCLR  4
; VSS   5 GND
; VDD  14 +5V
; OSC2 15 16.0 MHz crystal
; OSC1 16
;
; bits_byte,0 porta,0
; bits_byte,1 porta,1
; bits_byte,2 wind enable
; bits_byte,3 wind timeout
; bits_byte,4 speed of sound compare
; bits_byte,5 x, neg flag
; bits_byte,6 y, neg flag
; bits_byte,7 sonic virtual temperature, neg flag
;
; bits_b,0
; bits_b,1
; bits_b,2
; bits_b,3
; bits_b,4 carry
; bits_b,5
; bits_b,6
; bits_b,7 input negative
;
  include <p16f628a.inc>
;
MSD             EQU     0x020       ; Temporary register
LSD             EQU     0x021       ; Temporary register
TEMP0           EQU     0x022
TEMP1           EQU     0x023
TEMP20          EQU     0x024
TEMP21          EQU     0x025
TEMP22          EQU     0x026
TEMP23          EQU     0x027
TEMP30          EQU     0x028
TEMP31          EQU     0x029
TEMP32          EQU     0x02A
TEMP33          EQU     0x02B
DEC_S           EQU     0x02C
CNT1            EQU     0x02D
CNT2            EQU     0x02E
CNT3            EQU     0x02F
DIV1            EQU     0x030
DIV2            EQU     0x031
DIV3            EQU     0x032
DIV4            EQU     0x033
DIV5            EQU     0x034
DIV6            EQU     0x035
DIV7            EQU     0x036
DIV8            EQU     0x037
DIV9            EQU     0x038
DIV10           EQU     0x039
DIV11           EQU     0x03A
DIV12           EQU     0x03B
DIV13           EQU     0x03C
DIV14           EQU     0x03D
DIV15           EQU     0x03E
DIV16           EQU     0x03F
W11             EQU     0x040
W12             EQU     0x041
W21             EQU     0x042
W22             EQU     0x043
W31             EQU     0x044
W32             EQU     0x045
W41             EQU     0x046
W42             EQU     0x047
Y1              EQU     0x048
Y2              EQU     0x049
Y3              EQU     0x04A
Y4              EQU     0x04B
X1              EQU     0x04C
X2              EQU     0x04D
X3              EQU     0x04E
X4              EQU     0x04F
R1              EQU     0x050
R2              EQU     0x051
R3              EQU     0x052
DIR             EQU     0x053
CNT4            EQU     0x054
S_TEMP1         EQU     0x055
S_TEMP2         EQU     0x056
DIV17           EQU     0x057
DIV18           EQU     0x058
DIV19           EQU     0x059
DIV20           EQU     0x05A
SPD1            EQU     0x05B
SPD2            EQU     0x05C
SOS_Y0          EQU     0x05D
SOS_Y1          EQU     0x05E
SOS_Y2          EQU     0x05F
SOS_Y3          EQU     0x060
SOS_X0          EQU     0x061
SOS_X1          EQU     0x062
SOS_X2          EQU     0x063
SOS_X3          EQU     0x064
W_S             EQU     0x070   ; 70-7F accessed all banks
STATUS_S        EQU     0x071
FSR_SAVE        EQU     0x072
BITS_BYTE       EQU     0x073   ; bit buffer
BITS_B          EQU     0x074   ; bit buffer
WIND_CNT        EQU     0x075
W_TEMP          EQU     0x076
W_CRC           EQU     0x077
USART_D         EQU     0x078
TX_TEMP         EQU     0x079
FSR_S           EQU     0x07A
;
            org     0x0000          ; Reset vector location
RESET       CLRF    STATUS          ; Do initialization (Bank 0)
            CLRF    INTCON          ; Disable all interrupts
            BSF     STATUS, RP0     ; Bank 1
            GOTO    START
;
            org     0x0004          ; Interrupt vector location
INT_V       MOVWF   W_S             ; save reg's
            SWAPF   STATUS,W
            MOVWF   STATUS_S
            MOVF    FSR,W
            MOVWF   FSR_SAVE
            CLRF    STATUS          ; bank 0
;
TMR1_INT    BTFSS   PIR1,0
            GOTO    CCP1_INT
            BCF     PIR1,0          ; clear TMR1 int
            BTFSS   BITS_BYTE,2     ; wind enable
            GOTO    CCP1_INT
            BCF     BITS_BYTE,2
            BSF     BITS_BYTE,3     ; wind timeout
            MOVF    WIND_CNT,W
            MOVWF   FSR
            MOVLW   0xFF
            MOVWF   INDF
            INCF    FSR,F
            MOVWF   INDF
            INCF    FSR,W
            MOVWF   WIND_CNT
;
CCP1_INT    BTFSS   PIR1,2
            GOTO    FIN
            BCF     PIR1,2          ; clear CCP1 int
            BTFSS   BITS_BYTE,2     ; wind enable
            GOTO    FIN
            MOVF    WIND_CNT,W
            MOVWF   FSR
            MOVF    CCPR1H,W
            MOVWF   INDF
            INCF    FSR,F
            MOVF    CCPR1L,W
            MOVWF   INDF
            INCF    FSR,W
            MOVWF   WIND_CNT
            BCF     BITS_BYTE,2
;
FIN         MOVF    FSR_SAVE,W
            MOVWF   FSR
            SWAPF   STATUS_S,W      ; restore reg's
            MOVWF   STATUS
            SWAPF   W_S,F
            SWAPF   W_S,W
            RETFIE
;
START       MOVLW   0xE0
            MOVWF   TRISA           ; 4 - 0 outputs
            MOVLW   0xFF
            MOVWF   TRISB           ; 7 - 0 inputs
            MOVLW   0xC0
            MOVWF   OPTION_REG
            MOVLW   0xCF
            MOVWF   SPBRG           ; CF=4800 baud  67=9600 baud
            MOVLW   0xA7
            MOVWF   TXSTA
            CLRF    PIE1            ; disable interrupts
            BSF     PCON,0          ; brown-out reset
            BSF     PCON,1          ; power-on reset
            BCF     STATUS, RP0     ; Bank 0
            MOVLW   0x07
            MOVWF   CMCON           ; RA0 - RA3 digital input
            CLRF    CCP1CON
            MOVLW   0x90
            MOVWF   RCSTA           ; set receive
;
            CLRF    PORTA
            CLRF    USART_D
;
            MOVLW   0xC0
            MOVWF   INTCON
;
CHK_REC     BTFSS   PIR1,RCIF     ; check if data received
            GOTO    CHK_REC1
            MOVF    RCSTA,W
            ANDLW   0x06          ; error flags
            BTFSC   STATUS,Z
            GOTO    CAL           ; no error's
            MOVF    RCREG,W       ; clr framimg error bit
            BCF     RCSTA,4       ; clr overrun error bit
            NOP
            BSF     RCSTA,4
            GOTO    CHK_REC
;
CHK_REC1    MOVF    USART_D,W
            SUBLW   'L'
            BTFSS   STATUS,Z
            GOTO    CHK_REC
            GOTO    BEGIN
;
CAL         MOVF    RCREG,W       ; get received data into W
            MOVWF   USART_D
            SUBLW   'C'
            BTFSC   STATUS,Z
            GOTO    CAL2
            MOVF    USART_D,W
            SUBLW   'w'
            BTFSC   STATUS,Z
            GOTO    BEGIN
            MOVF    USART_D,W
            SUBLW   'W'
            BTFSC   STATUS,Z
            GOTO    BEGIN
            MOVF    USART_D,W
            SUBLW   'L'
            BTFSC   STATUS,Z
            GOTO    BEGIN
            MOVF    USART_D,W
            SUBLW   'V'
            BTFSS   STATUS,Z
            GOTO    CHK_REC
;
CAL1        MOVLW   0x00
            CALL    DATA_RD
            MOVWF   DIV2
            MOVLW   0x01
            CALL    DATA_RD
            MOVWF   DIV3
            MOVLW   0x02
            CALL    DATA_RD
            MOVWF   DIV4
            BCF     STATUS,C      ; x 2
            RLF     DIV4,F
            RLF     DIV3,F
            RLF     DIV2,F
            CALL    BIN_DEC_FR
            MOVLW   '0'
            CALL    RDY_TX
            MOVLW   '.'
            CALL    RDY_TX
            MOVF    DIV17,W
            CALL    DISP_BYTE
            MOVF    DIV18,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x08
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   0x09
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x0A
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   0x0B
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x0C
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   0x0D
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x0E
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   0x0F
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x10
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x11
            CALL    DATA_RD
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x12
            CALL    DATA_RD
            CALL    DISP_BYTE
            CALL    CAR_LN
            GOTO    CHK_REC
;
BEGIN       CALL    MEAS
;
            BTFSC   BITS_BYTE,3    ; wind timeout error
            GOTO    OUT_PUT0
;
            MOVLW   0x40
            MOVWF   FSR
            MOVLW   0x08             ; start of offset table
            MOVWF   WIND_CNT
;
APPLY_OFFS  MOVF    WIND_CNT,W
            CALL    DATA_RD
            MOVWF   DIV3
            INCF    WIND_CNT,F
            MOVF    WIND_CNT,W
            CALL    DATA_RD
            MOVWF   DIV4
            INCF    WIND_CNT,F
;
            MOVF    INDF,W
            MOVWF   DIV7
            INCF    FSR,F
            MOVF    INDF,W
            MOVWF   DIV8
            CLRF    DIV1
            CLRF    DIV2
            CLRF    DIV5
            CLRF    DIV6
            MOVF    FSR,W
            MOVWF   FSR_S
            CALL    SUB_7            ; add
;
            DECF    FSR_S,W
            MOVWF   FSR
            MOVF    DIV7,W
            MOVWF   INDF
            INCF    FSR,F
            MOVF    DIV8,W
            MOVWF   INDF
            INCF    FSR,F
;
            MOVLW   0x10
            SUBWF   WIND_CNT,W
            BTFSS   STATUS,Z
            GOTO    APPLY_OFFS
;
DISP        BCF     BITS_BYTE,6
            MOVF    W11,W
            MOVWF   DIV3
            MOVF    W12,W
            CALL    TIM
            CALL    TIM2
;
            MOVF    W31,W
            MOVWF   DIV3
            MOVF    W32,W
            CALL    TIM
            CALL    TIM3
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    TIM_31         ; C=1 on positive result
            MOVLW   0x34
            CALL    SUB_64
            BSF     BITS_BYTE,6    ; negative
TIM_31      CALL    TIM4
;
            MOVF    DIV6,W
            MOVWF   Y1
            MOVF    DIV7,W
            MOVWF   Y2
            MOVF    DIV8,W
            MOVWF   Y3
            MOVF    DIV9,W
            MOVWF   Y4
;
            CALL    SOS_CAL
;
            MOVF    DIV5,W
            MOVWF   SOS_Y0         ; save speed of sound Y axis
            MOVF    DIV6,W
            MOVWF   SOS_Y1
            MOVF    DIV7,W
            MOVWF   SOS_Y2
            MOVF    DIV8,W
            MOVWF   SOS_Y3
;
            BCF     BITS_BYTE,5
            MOVF    W21,W
            MOVWF   DIV3
            MOVF    W22,W
            CALL    TIM
            CALL    TIM2
;
            MOVF    W41,W
            MOVWF   DIV3
            MOVF    W42,W
            CALL    TIM
            CALL    TIM3
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    TIM_32         ; C=1 on positive result
            MOVLW   0x34
            CALL    SUB_64
            BSF     BITS_BYTE,5    ; negative
TIM_32      CALL    TIM4
;
            MOVF    DIV6,W
            MOVWF   X1
            MOVF    DIV7,W
            MOVWF   X2
            MOVF    DIV8,W
            MOVWF   X3
            MOVF    DIV9,W
            MOVWF   X4
;
            CALL    SOS_CAL
;
            MOVF    DIV5,W
            MOVWF   SOS_X0         ; save speed of sound X axis
            MOVWF   DIV1
            MOVWF   DIV13
            MOVF    DIV6,W
            MOVWF   SOS_X1
            MOVWF   DIV2
            MOVWF   DIV14
            MOVF    DIV7,W
            MOVWF   SOS_X2
            MOVWF   DIV3
            MOVWF   DIV15
            MOVF    DIV8,W
            MOVWF   SOS_X3
            MOVWF   DIV4
            MOVWF   DIV16
            CALL    MULT
;
            MOVF    DIV9,W
            MOVWF   DIV12
            MOVF    DIV6,W
            MOVWF   DIV9
            MOVF    DIV7,W
            MOVWF   DIV10
            MOVF    DIV8,W
            MOVWF   DIV11
;
            MOVLW   0x01           ; 401.856 = 1.4 * 287.04
            MOVWF   DIV1
            MOVLW   0x91
            MOVWF   DIV2
            MOVLW   0xDB
            MOVWF   DIV3
            MOVLW   0x22
            MOVWF   DIV4
            CALL    DIV_BIN00
;
            MOVF    DIV11,W
            MOVWF   DIV5
            MOVF    DIV12,W
            MOVWF   DIV6
            MOVF    DIV13,W
            MOVWF   DIV7
            MOVF    DIV14,W
            MOVWF   DIV8
;
            CLRF    DIV1
            MOVLW   0x01           ; 273.15
            MOVWF   DIV2
            MOVLW   0x11
            MOVWF   DIV3
            MOVLW   0x26
            MOVWF   DIV4
            CALL    SUB_6
            CALL    SUB_7
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    TIM_33         ; C=1 on positive result
            MOVLW   0x34
            CALL    SUB_64
            BSF     BITS_BYTE,7    ; negative
TIM_33      MOVF    DIV7,W
            MOVWF   S_TEMP1
            MOVF    DIV8,W
            MOVWF   S_TEMP2
;
            MOVF    S_TEMP1,W
            MOVWF   DIV10
            CALL    HEX_DEC
            MOVF    DIV8,W
            MOVWF   S_TEMP1
            MOVF    S_TEMP2,W
            MOVWF   DIV2
            CLRF    DIV3
            CLRF    DIV4
            CALL    BIN_DEC_FR
            MOVF    DIV17,W
            MOVWF   S_TEMP2
;
            MOVF    X1,W           ; square
            MOVWF   DIV1
            MOVWF   DIV13
            MOVF    X2,W
            MOVWF   DIV2
            MOVWF   DIV14
            MOVF    X3,W
            MOVWF   DIV3
            MOVWF   DIV15
            MOVF    X4,W
            MOVWF   DIV4
            MOVWF   DIV16
            CALL    MULT
;
            MOVF    DIV5,W
            MOVWF   TEMP20
            MOVF    DIV6,W
            MOVWF   TEMP21
            MOVF    DIV7,W
            MOVWF   TEMP22
            MOVF    DIV8,W
            MOVWF   TEMP23
;
            MOVF    Y1,W           ; square
            MOVWF   DIV1
            MOVWF   DIV13
            MOVF    Y2,W
            MOVWF   DIV2
            MOVWF   DIV14
            MOVF    Y3,W
            MOVWF   DIV3
            MOVWF   DIV15
            MOVF    Y4,W
            MOVWF   DIV4
            MOVWF   DIV16
            CALL    MULT
;
            MOVF    TEMP20,W
            MOVWF   DIV1
            MOVF    TEMP21,W
            MOVWF   DIV2
            MOVF    TEMP22,W
            MOVWF   DIV3
            MOVF    TEMP23,W
            MOVWF   DIV4
;
            CALL    SUB_7          ; add
;
            MOVF    DIV5,W
            MOVWF   DIV9
            MOVF    DIV6,W
            MOVWF   DIV10
            MOVF    DIV7,W
            MOVWF   DIV11
            MOVF    DIV8,W
            MOVWF   DIV12
;
            CALL    SQRT_BIN
;
            MOVF    DIV13,W
            MOVWF   R1
            MOVF    DIV14,W
            MOVWF   R2
            MOVF    DIV15,W
            MOVWF   R3
;
            MOVF    Y1,W
            MOVWF   DIV9
            MOVF    Y2,W
            MOVWF   DIV10
            MOVF    Y3,W
            MOVWF   DIV11
            MOVF    Y4,W
            MOVWF   DIV12
            CLRF    DIV1
            MOVF    R1,W
            MOVWF   DIV2
            MOVF    R2,W
            MOVWF   DIV3
            MOVF    R3,W
            MOVWF   DIV4
            CALL    DIV_BIN00
            MOVF    DIV11,W
            MOVWF   DIV5
            MOVF    DIV12,W
            MOVWF   DIV6
            MOVF    DIV13,W
            MOVWF   DIV7
            MOVF    DIV14,W
            MOVWF   DIV8
            CALL    ARCSIN
;
            CLRF    DIV1            ; convert to Km/Hr
            CLRF    DIV2
            MOVF    R1,W
            MOVWF   DIV3
            MOVF    R2,W
            MOVWF   DIV4
            MOVLW   0x03            ; 3.6
            MOVWF   DIV13
            MOVLW   0x99
            MOVWF   DIV14
            MOVWF   DIV15
            MOVWF   DIV16
            CALL    MULT
;
            MOVF    DIV7,W
            MOVWF   DIV9
            MOVF    DIV8,W
            MOVWF   DIV10
            CALL    HEX_DEC_0     ; convert to decimal
            MOVF    DIV7,W
            MOVWF   SPD1
            MOVF    DIV8,W
            MOVWF   SPD2
;
            MOVF    SOS_Y0,W
            MOVWF   DIV1
            MOVF    SOS_Y1,W
            MOVWF   DIV2
            MOVF    SOS_Y2,W
            MOVWF   DIV3
            MOVF    SOS_Y3,W
            MOVWF   DIV4
;
            MOVF    SOS_X0,W
            MOVWF   DIV5
            MOVF    SOS_X1,W
            MOVWF   DIV6
            MOVF    SOS_X2,W
            MOVWF   DIV7
            MOVF    SOS_X3,W
            MOVWF   DIV8
;
            BCF     BITS_BYTE,4
            CALL    SUB_6
            CALL    SUB_7
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    CHK_RT         ; C=1 on positive result
            MOVLW   0x34
            CALL    SUB_64
CHK_RT      MOVF    DIV6,W
            BTFSS   STATUS,Z
            BSF     BITS_BYTE,4
;
OUT_PUT0    MOVF    USART_D,W
            SUBLW   'w'
            BTFSC   STATUS,Z
            GOTO    OUT_PUT
            MOVF    USART_D,W
            SUBLW   'W'
            BTFSC   STATUS,Z
            GOTO    OUT_PUT1
            MOVF    USART_D,W
            SUBLW   'L'
            BTFSC   STATUS,Z
            GOTO    OUT_PUT1
            GOTO    CHK_REC
;
OUT_PUT     CLRF    W_CRC
            MOVF    BITS_BYTE,W
            CALL    CRC
;
            SWAPF   DIR,W
            IORWF   SPD1,W
            CALL    CRC
            MOVF    SPD2,W
            CALL    CRC
            MOVF    S_TEMP1,W
            CALL    CRC
            MOVF    S_TEMP2,W
            CALL    CRC
;
            MOVF    W_CRC,W
            CALL    RDY_TX
            CALL    DELAY2
            GOTO    CHK_REC
;
OUT_PUT1    MOVLW   'T'
            BTFSC   BITS_BYTE,3
            GOTO    OUT_PUT2
            MOVLW   ' '
            BTFSC   BITS_BYTE,4
            MOVLW   'E'
OUT_PUT2    CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x40
            CALL    MOVE_S
            MOVLW   0x42
            CALL    MOVE_S
            MOVLW   0x44
            CALL    MOVE_S
            MOVLW   0x46
            CALL    MOVE_S
;
            BTFSC   BITS_BYTE,3
            GOTO    OUT_PUT3
            BTFSC   BITS_BYTE,4
            GOTO    OUT_PUT3
;
            MOVLW   '+'
            BTFSC   BITS_BYTE,5
            MOVLW   '-'
            CALL    RDY_TX
            MOVF    X1,W
            MOVWF   DIV10
            CALL    HEX_DEC
            MOVF    DIV7,W
            CALL    DISP_B_LO
            MOVF    DIV8,W
            CALL    DISP_BYTE
            MOVLW   '.'
            CALL    RDY_TX
            MOVF    X2,W
            MOVWF   DIV2
            MOVF    X3,W
            MOVWF   DIV3
            MOVF    X4,W
            MOVWF   DIV4
            CALL    BIN_DEC_FR
            MOVF    DIV17,W
            CALL    DISP_BYTE
            MOVF    DIV18,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   '+'
            BTFSC   BITS_BYTE,6
            MOVLW   '-'
            CALL    RDY_TX
            MOVF    Y1,W
            MOVWF   DIV10
            CALL    HEX_DEC
            MOVF    DIV7,W
            CALL    DISP_B_LO
            MOVF    DIV8,W
            CALL    DISP_BYTE
            MOVLW   '.'
            CALL    RDY_TX
            MOVF    Y2,W
            MOVWF   DIV2
            MOVF    Y3,W
            MOVWF   DIV3
            MOVF    Y4,W
            MOVWF   DIV4
            CALL    BIN_DEC_FR
            MOVF    DIV17,W
            CALL    DISP_BYTE
            MOVF    DIV18,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVF    R1,W
            MOVWF   DIV10
            CALL    HEX_DEC
            MOVF    DIV7,W
            CALL    DISP_B_LO
            MOVF    DIV8,W
            CALL    DISP_BYTE
            MOVLW   '.'
            CALL    RDY_TX
            MOVF    R2,W
            MOVWF   DIV2
            MOVF    R3,W
            MOVWF   DIV3
            CLRF    DIV4
            CALL    BIN_DEC_FR
            MOVF    DIV17,W
            CALL    DISP_BYTE
            MOVF    DIV18,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
DISP1       CLRF    CNT2
            MOVF    DIR,W
            BTFSC   STATUS,Z
            GOTO    NEXT11_3
            MOVWF   CNT1
NEXT11_2    MOVLW   0x03
            ADDWF   CNT2,F
            DECFSZ  CNT1,F
            GOTO    NEXT11_2

NEXT11_3    MOVLW   0x03
            MOVWF   CNT1
NEXT11_4    MOVLW   0x02
            MOVWF   PCLATH          ; bits 0-2, each 256 byte block
            MOVF    CNT2,W
            CALL    WIND_DIR
            CALL    RDY_TX
            INCF    CNT2,F
            DECFSZ  CNT1,F
            GOTO    NEXT11_4
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVF    SPD1,W
            CALL    DISP_B_LO
            MOVF    SPD2,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   '+'
            BTFSC   BITS_BYTE,7
            MOVLW   '-'
            CALL    RDY_TX
            MOVF    S_TEMP1,W
            CALL    DISP_BYTE
            MOVLW   '.'
            CALL    RDY_TX
            SWAPF   S_TEMP2,W
            CALL    DISP_B_LO
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVF    SOS_X0,W
            MOVWF   DIV9
            MOVF    SOS_X1,W
            MOVWF   DIV10
            CALL    HEX_DEC_0
            MOVF    DIV7,W
            CALL    DISP_B_LO
            MOVF    DIV8,W
            CALL    DISP_BYTE
            MOVLW   '.'
            CALL    RDY_TX
            MOVF    SOS_X2,W
            MOVWF   DIV2
            MOVF    SOS_X3,W
            MOVWF   DIV3
            CLRF    DIV4
            CALL    BIN_DEC_FR
            MOVF    DIV17,W
            CALL    DISP_BYTE
;
OUT_PUT3    CALL    CAR_LN
            CALL    DELAY2
            GOTO    CHK_REC
;
CRC         MOVWF   W_TEMP
            ADDWF   W_CRC, F
            BTFSC   STATUS, C
            INCF    W_CRC, F
            MOVF    W_TEMP, W
            CALL    RDY_TX
            RETURN
;
WIND_DIR    ADDWF   PCL,F
            RETLW   ' '
            RETLW   ' '
            RETLW   'N'

            RETLW   'N'
            RETLW   'N'
            RETLW   'E'

            RETLW   ' '
            RETLW   'N'
            RETLW   'E'

            RETLW   'E'
            RETLW   'N'
            RETLW   'E'

            RETLW   ' '
            RETLW   ' '
            RETLW   'E'

            RETLW   'E'
            RETLW   'S'
            RETLW   'E'

            RETLW   ' '
            RETLW   'S'
            RETLW   'E'

            RETLW   'S'
            RETLW   'S'
            RETLW   'E'

            RETLW   ' '
            RETLW   ' '
            RETLW   'S'

            RETLW   'S'
            RETLW   'S'
            RETLW   'W'

            RETLW   ' '
            RETLW   'S'
            RETLW   'W'

            RETLW   'W'
            RETLW   'S'
            RETLW   'W'

            RETLW   ' '
            RETLW   ' '
            RETLW   'W'

            RETLW   'W'
            RETLW   'N'
            RETLW   'W'

            RETLW   ' '
            RETLW   'N'
            RETLW   'W'

            RETLW   'N'
            RETLW   'N'
            RETLW   'W'
;
CAL2        CALL    ENTR
            MOVLW   'D'
            CALL    RDY_TX
            MOVLW   'i'
            CALL    RDY_TX
            MOVLW   's'
            CALL    RDY_TX
            MOVLW   't'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
            MOVLW   '0'
            CALL    RDY_TX
            MOVLW   '.'
            CALL    RDY_TX
;
            CALL    REC_DATA
            MOVWF   DIV6
            CALL    REC_DATA
            MOVWF   DIV7
            CLRF    DIV8
            CALL    DEC_BIN_FR
;
            BCF     STATUS,C        ; /2
            RRF     DIV9,F
            RRF     DIV10,F
            RRF     DIV11,F
            RRF     DIV12,F
;
            MOVF    DIV9,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x00
            CALL    DATA_WR         ; 4 - 8ms
;
            MOVF    DIV10,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x01
            CALL    DATA_WR
;
            MOVF    DIV11,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x02
            CALL    DATA_WR
;
            MOVF    DIV12,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x03
            CALL    DATA_WR
            CALL    CAR_LN
;
            CALL    ENTR
            MOVLW   'P'
            CALL    RDY_TX
            MOVLW   'u'
            CALL    RDY_TX
            MOVLW   'l'
            CALL    RDY_TX
            MOVLW   's'
            CALL    RDY_TX
            MOVLW   'e'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            CALL    REC_DATA
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x10
            CALL    DATA_WR
            CALL    CAR_LN
;
            CALL    ENTR
            MOVLW   'D'
            CALL    RDY_TX
            MOVLW   'e'
            CALL    RDY_TX
            MOVLW   'l'
            CALL    RDY_TX
            MOVLW   'a'
            CALL    RDY_TX
            MOVLW   'y'
            CALL    RDY_TX
            MOVLW   '1'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            CALL    REC_DATA
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x11
            CALL    DATA_WR
            CALL    CAR_LN
;
            CALL    ENTR
            MOVLW   'D'
            CALL    RDY_TX
            MOVLW   'e'
            CALL    RDY_TX
            MOVLW   'l'
            CALL    RDY_TX
            MOVLW   'a'
            CALL    RDY_TX
            MOVLW   'y'
            CALL    RDY_TX
            MOVLW   '2'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            CALL    REC_DATA
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x12
            CALL    DATA_WR
            CALL    CAR_LN
;
ADJ_TRIM    CALL    MEAS
            MOVLW   'T'
            BTFSS   BITS_BYTE,3      ; wind timeout
            MOVLW   ' '
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   0x40
            CALL    MOVE_S
            MOVLW   0x42
            CALL    MOVE_S
            MOVLW   0x44
            CALL    MOVE_S
            MOVLW   0x46
            CALL    MOVE_S
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVF    W11,W
            MOVWF   DIV3
            MOVF    W12,W
            MOVWF   DIV4
            MOVF    W31,W
            MOVWF   DIV7
            MOVF    W32,W
            CALL    DIFF_CNT
;
            MOVF    W21,W
            MOVWF   DIV3
            MOVF    W22,W
            MOVWF   DIV4
            MOVF    W41,W
            MOVWF   DIV7
            MOVF    W42,W
            CALL    DIFF_CNT
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   'A'
            CALL    RDY_TX
            MOVLW   'd'
            CALL    RDY_TX
            MOVLW   'j'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   'T'
            CALL    RDY_TX
            MOVLW   'r'
            CALL    RDY_TX
            MOVLW   'i'
            CALL    RDY_TX
            MOVLW   'm'
            CALL    RDY_TX
            MOVLW   'p'
            CALL    RDY_TX
            MOVLW   'o'
            CALL    RDY_TX
            MOVLW   't'
            CALL    RDY_TX
            MOVLW   ','
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   'H'
            CALL    RDY_TX
            MOVLW   'i'
            CALL    RDY_TX
            MOVLW   't'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   'A'
            CALL    RDY_TX
            MOVLW   'n'
            CALL    RDY_TX
            MOVLW   'y'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   'K'
            CALL    RDY_TX
            MOVLW   'e'
            CALL    RDY_TX
            MOVLW   'y'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   'T'
            CALL    RDY_TX
            MOVLW   'o'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVLW   'E'
            CALL    RDY_TX
            MOVLW   'x'
            CALL    RDY_TX
            MOVLW   'i'
            CALL    RDY_TX
            MOVLW   't'
            CALL    RDY_TX
            CALL    CAR_LN
;
            BTFSS   PIR1,RCIF     ; check if data received
            GOTO    ADJ_TRIM
            MOVF    RCSTA,W
            ANDLW   0x06          ; error flags
            BTFSC   STATUS,Z
            GOTO    CAL4          ; no error's
            MOVF    RCREG,W       ; clr framimg error bit
            BCF     RCSTA,4       ; clr overrun error bit
            NOP
            BSF     RCSTA,4
            GOTO    ADJ_TRIM
;
CAL4        MOVF    RCREG,W       ; get received data into W
            CALL    ENTR
            MOVLW   'T'
            CALL    RDY_TX
            MOVLW   'e'
            CALL    RDY_TX
            MOVLW   'm'
            CALL    RDY_TX
            MOVLW   'p'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
;
            BCF     BITS_B,7
            CALL    REC_DATA
            MOVWF   DIV8
            MOVLW   '.'
            CALL    RDY_TX
            CALL    REC_DATA
            MOVWF   DIV9
;
            CALL    DEC_BIN        ; decimal to binary
            MOVF    DIV9,W
            MOVWF   DIV6
            CLRF    DIV7
            CLRF    DIV8
            CALL    DEC_BIN_FR
            MOVF    DIV9,W
            MOVWF   DIV4
            MOVF    DIV16,W
            MOVWF   DIV3
;
            CLRF    DIV1
            CLRF    DIV2
            CLRF    DIV5
            MOVLW   0x01           ; 273.15
            MOVWF   DIV6
            MOVLW   0x11
            MOVWF   DIV7
            MOVLW   0x26
            MOVWF   DIV8
            BTFSC   BITS_B,7       ; number negative
            CALL    SUB_6
            CALL    SUB_7
;
            MOVF    DIV5,W
            MOVWF   DIV13
            MOVF    DIV6,W
            MOVWF   DIV14
            MOVF    DIV7,W
            MOVWF   DIV15
            MOVF    DIV8,W
            MOVWF   DIV16
            MOVLW   0x01           ; 401.856 = 1.4 * 287.04
            MOVWF   DIV1
            MOVLW   0x91
            MOVWF   DIV2
            MOVLW   0xDB
            MOVWF   DIV3
            MOVLW   0x22
            MOVWF   DIV4
            CALL    MULT
;
            MOVF    DIV9,W
            MOVWF   DIV11
            MOVF    DIV10,W
            MOVWF   DIV12
            MOVF    DIV7,W
            MOVWF   DIV9
            MOVF    DIV8,W
            MOVWF   DIV10
            CALL    SQRT_BIN
;
            MOVF    DIV13,W         ; temp save
            MOVWF   DIV17
            MOVF    DIV14,W
            MOVWF   DIV18
            MOVF    DIV15,W
            MOVWF   DIV19
;
            CALL    CAL_DIST
;
            MOVLW   ' '
            CALL    RDY_TX
            MOVF    DIV11,W
            CALL    DISP_BYTE
            MOVF    DIV12,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
;
            MOVF    W11,W
            MOVWF   DIV3
            MOVF    W12,W
            CALL    CAL5
;
            MOVLW   0x08
            CALL    DATA_WR
;
            MOVF    DIV8,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x09
            CALL    DATA_WR
;
            MOVF    W21,W
            MOVWF   DIV3
            MOVF    W22,W
            CALL    CAL5
;
            MOVLW   0x0A
            CALL    DATA_WR
;
            MOVF    DIV8,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x0B
            CALL    DATA_WR
;
            MOVF    W31,W
            MOVWF   DIV3
            MOVF    W32,W
            CALL    CAL5
;
            MOVLW   0x0C
            CALL    DATA_WR
;
            MOVF    DIV8,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x0D
            CALL    DATA_WR
;
            MOVF    W41,W
            MOVWF   DIV3
            MOVF    W42,W
            CALL    CAL5
;
            MOVLW   0x0E
            CALL    DATA_WR
;
            MOVF    DIV8,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            MOVLW   0x0F
            CALL    DATA_WR
;
            CLRF    DIV16
            MOVLW   0x04            ; move 4 numbers to left, 4 places
            MOVWF   CNT2
MOVE_L      BCF     STATUS,C
            RLF     DIV19,F
            RLF     DIV18,F
            RLF     DIV17,F
            RLF     DIV16,F
            DECFSZ  CNT2,F
            GOTO    MOVE_L
;
            MOVF    DIV16,W
            MOVWF   DIV9
            MOVF    DIV17,W
            MOVWF   DIV10
            CALL    HEX_DEC_0
            MOVF    DIV7,W
            CALL    DISP_B_LO
            MOVF    DIV8,W
            CALL    DISP_BYTE
            MOVLW   '.'
            CALL    RDY_TX
;
            MOVF    DIV18,W
            MOVWF   DIV2
            MOVF    DIV19,W
            MOVWF   DIV3
            CLRF    DIV4
            CALL    BIN_DEC_FR
            MOVF    DIV17,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
            MOVLW   'm'
            CALL    RDY_TX
            MOVLW   '/'
            CALL    RDY_TX
            MOVLW   's'
            CALL    RDY_TX
;
            CALL    CAR_LN
            GOTO    CHK_REC
;
DIFF_CNT    MOVWF   DIV8
            CLRF    DIV1
            CLRF    DIV2
            CLRF    DIV5
            CLRF    DIV6
            CALL    SUB_6
            CALL    SUB_7
            MOVF    DIV7,W
            CALL    DISP_BYTE
            MOVF    DIV8,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
            RETURN
;
SOS_CAL     MOVF    TEMP20,W         ; cal speed of sound
            MOVWF   DIV1
            MOVF    TEMP21,W
            MOVWF   DIV2
            MOVF    TEMP22,W
            MOVWF   DIV3
            MOVF    TEMP23,W
            MOVWF   DIV4
;
            MOVF    TEMP30,W
            MOVWF   DIV5
            MOVF    TEMP31,W
            MOVWF   DIV6
            MOVF    TEMP32,W
            MOVWF   DIV7
            MOVF    TEMP33,W
            MOVWF   DIV8
            CALL    SUB_7            ; add
            CALL    TIM4
            RETURN
;
MEAS        BCF     STATUS, RP0      ; Bank 0
            BSF     PORTA,3          ; sync for CRO trig input
            CLRF    BITS_BYTE        ; clear bit 3
            MOVLW   0x40
            MOVWF   WIND_CNT
;
SCAN        MOVF    BITS_BYTE,W
            ANDLW   0x03
            MOVWF   PORTA
;
            MOVLW   0x10
            CALL    DATA_RD
            MOVWF   MSD              ; number of cycles of 40kHz
;
            CLRF    TMR1H
            CLRF    TMR1L
;
            MOVLW   0x01
            MOVWF   T1CON            ; internal clk, enables timer 1
            MOVLW   0x04
            MOVWF   CCP1CON          ; capture mode, falling edge
; 
LOOP6       BSF     PORTA,2
            MOVLW   0x10
            MOVWF   LSD 
LOOP4       DECFSZ  LSD,F            ; 12.5us = ((LSD - 1) * 3 + 5) * Tcy
            GOTO    LOOP4
            BCF     PORTA,2
            MOVLW   0x0F
            MOVWF   LSD 
LOOP5       DECFSZ  LSD,F            ; 12.5us = ((LSD - 1) * 3 + 8) * Tcy
            GOTO    LOOP5
            DECFSZ  MSD,F
            GOTO    LOOP6
;
            MOVLW   0x11
            CALL    DATA_RD
            MOVWF   LSD              ; delay, wait for ringing to stop
LOOP3       NOP
            NOP
            NOP
            NOP
            NOP
            NOP
            DECFSZ  LSD,F            ; Delay time = ((LSD - 1) * 9 + 18) * Tcy
            GOTO    LOOP3
;
            BCF     PIR1,0           ; clr TMR1 int
            BCF     PIR1,2           ; clr CCP1 int
            BSF     STATUS, RP0      ; Bank 1
            BSF     PIE1,0           ; enable TMR1 int
            BSF     PIE1,2           ; enable CCP1 int
            BCF     STATUS, RP0      ; Bank 0
            BSF     BITS_BYTE,2
;
READ_WIND   BTFSC   BITS_BYTE,2      ; wind enable
            GOTO    READ_WIND
;
            CLRF    CCP1CON
            BCF     T1CON,0          ; stops timer 1
            BSF     STATUS, RP0      ; Bank 1
            BCF     PIE1,0           ; disable TMR1 int
            BCF     PIE1,2           ; disable CCP1 int
            BCF     STATUS, RP0      ; Bank 0
;
            MOVLW   0x03
            ANDWF   BITS_BYTE,W
            SUBLW   0x03
            BTFSC   STATUS,Z
            RETURN
            CALL    DELAY2
            INCF    BITS_BYTE,F
            GOTO    SCAN
;
CAL_DIST    MOVLW   0x7A            ; clk x 2 = 8000000
            MOVWF   DIV2
            MOVLW   0x12
            MOVWF   DIV3
            CLRF    DIV4
            CLRF    DIV1
            CALL    DIST            ; 0.1 = 0.2 / 2
;
            CLRF    DIV1
            MOVF    DIV17,W
            MOVWF   DIV2
            MOVF    DIV18,W
            MOVWF   DIV3
            MOVF    DIV19,W
            MOVWF   DIV4
;
            MOVF    DIV6,W
            MOVWF   DIV9
            MOVF    DIV7,W
            MOVWF   DIV10
            MOVF    DIV8,W
            MOVWF   DIV11
            CALL    DIV_BIN0
;
            MOVLW   0x04            ; move 4 numbers to left, 4 places
            MOVWF   CNT2
CAL7        BCF     STATUS,C
            RLF     DIV14,F
            RLF     DIV13,F
            RLF     DIV12,F
            RLF     DIV11,F
            DECFSZ  CNT2,F
            GOTO    CAL7
            RETURN
;
CAL5        MOVWF   DIV4
            CLRF    DIV1
            CLRF    DIV2
            CLRF    DIV5
            CLRF    DIV6
            MOVF    DIV11,W
            MOVWF   DIV7
            MOVF    DIV12,W
            MOVWF   DIV8
            CALL    SUB_6
            CALL    SUB_7
;
            MOVF    DIV7,W
            BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEDATA
            RETURN
;
CAR_LN      MOVLW   0x0D          ; carriage return
            CALL    RDY_TX      
            MOVLW   0x0A          ; line feed
            CALL    RDY_TX
            RETURN
;
ENTR        MOVLW   'E'
            CALL    RDY_TX
            MOVLW   'n'
            CALL    RDY_TX
            MOVLW   't'
            CALL    RDY_TX
            MOVLW   'e'
            CALL    RDY_TX
            MOVLW   'r'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
            RETURN
;
REC_DATA    BTFSS   PIR1,RCIF     ; check if data received
            GOTO    REC_DATA
            MOVF    RCSTA,W
            ANDLW   0x06          ; error flags
            BTFSC   STATUS,Z
            GOTO    REC_DATA0     ; no error's
            MOVF    RCREG,W       ; clr framimg error bit
            BCF     RCSTA,4       ; clr overrun error bit
            NOP
            BSF     RCSTA,4
            GOTO    REC_DATA
;
REC_DATA0   MOVF    RCREG,W
            MOVWF   TEMP0
            CALL    RDY_TX
            MOVF    TEMP0,W
            SUBLW   '-'
            BTFSS   STATUS,Z
            GOTO    REC_DATA1
            BSF     BITS_B,7
            GOTO    REC_DATA
;
REC_DATA1   MOVF    TEMP0,W
            ADDLW   0xBF          ; add -'A' to Ascii high byte
            BTFSS   STATUS,C      ; check if positive
            ADDLW   0x07          ; if not, add 7 ('0' to '9')
            ADDLW   0x0A          ; else add 10 ('A' to 'F') 
            MOVWF   TEMP1
            SWAPF   TEMP1,F       ; move nibble to high position
;
REC_DATA2   BTFSS   PIR1,RCIF     ; check if data received
            GOTO    REC_DATA2
            MOVF    RCSTA,W
            ANDLW   0x06          ; error flags
            BTFSC   STATUS,Z
            GOTO    REC_DATA20    ; no error's
            MOVF    RCREG,W       ; clr framimg error bit
            BCF     RCSTA,4       ; clr overrun error bit
            NOP
            BSF     RCSTA,4
            GOTO    REC_DATA2
;
REC_DATA20  MOVF    RCREG,W
            MOVWF   TEMP0
            CALL    RDY_TX
            MOVF    TEMP0,W
            ADDLW   0xBF          ; add -'A' to Ascii high byte
            BTFSS   STATUS,C      ; check if positive
            ADDLW   0x07          ; if not, add 7 ('0' to '9')
            ADDLW   0x0A          ; else add 10 ('A' to 'F') 
            IORWF   TEMP1,W       ; add low nibble to high nibble
            RETURN
;
DATA_WR     MOVWF   EEADR
            BCF     INTCON,7        ; disable int's
            BSF     EECON1,WREN     ; enable write
            MOVLW   0x55
            MOVWF   EECON2
            MOVLW   0xAA
            MOVWF   EECON2
            BSF     EECON1,WR
CHK_WR      BTFSC   EECON1,WR
            GOTO    CHK_WR
            BCF     EECON1,WREN
            BSF     INTCON,7
            BTFSC   EECON1,3        ; write error
            GOTO    DATA_WR1
            BCF     STATUS, RP0     ; Bank 0
            RETURN
;
DATA_WR1    BCF     STATUS, RP0     ; Bank 0
            MOVLW   ' '
            CALL    RDY_TX
            MOVLW   'M'
            CALL    RDY_TX
            MOVLW   'e'
            CALL    RDY_TX
            MOVLW   'm'
            CALL    RDY_TX
            GOTO    DISP_F
;
DATA_RD     BSF     STATUS, RP0     ; Bank 1
            MOVWF   EEADR
            BSF     EECON1,RD
            MOVF    EEDATA,W
            BCF     STATUS, RP0     ; Bank 0
            RETURN
;
DISP_F      MOVLW   ' '
            CALL    RDY_TX
            MOVLW   'F'
            CALL    RDY_TX
            MOVLW   'a'
            CALL    RDY_TX
            MOVLW   'u'
            CALL    RDY_TX
            MOVLW   'l'
            CALL    RDY_TX
            MOVLW   't'
            CALL    RDY_TX
            MOVLW   ' '
            CALL    RDY_TX
            RETURN
;
DELAY2      MOVLW   0x12         ; delay
            CALL    DATA_RD
SLOW        MOVWF   MSD
            CLRF    LSD
LOOP2       DECFSZ  LSD, F       ; ((MSD * 765) + ((MSD - 1) * 3) + 15) * Tcy
            GOTO    LOOP2
            DECFSZ  MSD, F
            GOTO    LOOP2
            RETURN
;
MOVE_S      MOVWF   FSR
            MOVF    INDF,W
            CALL    DISP_BYTE
            INCF    FSR,F
            MOVF    INDF,W
            CALL    DISP_BYTE
            MOVLW   ' '
            CALL    RDY_TX
            RETURN
;
DISP_BYTE   MOVWF   TEMP1
            SWAPF   TEMP1, W 
            CALL    HEX_ASCII
            CALL    RDY_TX
            MOVF    TEMP1, W
DISP_B_LO   CALL    HEX_ASCII
            CALL    RDY_TX
            RETURN
;
RDY_TX      BCF     STATUS, RP0     ; Bank 0
            MOVWF   TX_TEMP
            MOVLW   0xFA            ; give receive end time
            MOVWF   LSD
LOOP99      NOP
            DECFSZ  LSD,F           ; 250us = ((LSD * 4) + 3) * Tcy
            GOTO    LOOP99
;
RDY_TX1     BTFSS   PIR1,4          ; transmit buffer empty  TXSTA,1
            GOTO    RDY_TX1
            MOVF    TX_TEMP,W
            MOVWF   TXREG
            RETURN
;
HEX_ASCII   ANDLW   0x0F
            MOVWF   TEMP0
            SUBLW   0x09
            BTFSC   STATUS, C
            GOTO    FALSE
            MOVF    TEMP0, W
            ADDLW   0x07
            GOTO    FALSE2
FALSE       MOVF    TEMP0, W
FALSE2      ADDLW   0x30            ;set to correct ascii
            RETURN
;
SQRT_BIN    CLRF    DIV5            ; square root binary
            CLRF    DIV6
            CLRF    DIV7
            CLRF    DIV8
            CLRF    DIV13
            CLRF    DIV14
            CLRF    DIV15
            MOVLW   0x18            ; move 24 times
            MOVWF   CNT1
SQRT_BIN5   CALL    SQRT_BIN40
            CLRF    DIV1
            MOVF    DIV13,W
            MOVWF   DIV2
            MOVF    DIV14,W
            MOVWF   DIV3
            MOVF    DIV15,W
            MOVWF   DIV4
            BSF     STATUS,C        ; set carry - 01
            RLF     DIV4,F
            RLF     DIV3,F
            RLF     DIV2,F
            RLF     DIV1,F
            CALL    SUB_6
            CALL    SUB_7
            BTFSS   STATUS,C       ; C=1 on positive result
            GOTO    SQRT_BIN6      ; C=0 on negative result
            INCF    DIV15,F
            GOTO    SQRT_BIN7
SQRT_BIN6   CALL    SUB_6          ; return to original value
            CALL    SUB_7          ; return to original value
SQRT_BIN7   DECFSZ  CNT1,F
            GOTO    SQRT_BIN5
            RETURN
;
SQRT_BIN40  MOVLW   0x02
            MOVWF   CNT2
SQRT_BIN41  MOVLW   0x08            ; move 8 numbers to left, 2 places
            MOVWF   CNT3
            MOVLW   0x3C
            MOVWF   FSR
            CALL    DIV_5
            BTFSS   STATUS, Z
            GOTO    SQRT_BIN41
            BCF     STATUS,C        ; clear carry
            RLF     DIV15,F
            RLF     DIV14,F
            RLF     DIV13,F
            RETURN
;
TIM         MOVWF   DIV4
            MOVLW   0x3D            ; clk = 4000000
            MOVWF   DIV9
            MOVLW   0x09
            MOVWF   DIV10
            CLRF    DIV11
            CLRF    DIV1
            CLRF    DIV2
            CALL    DIV_BIN0
            RETURN
;
TIM2        MOVF    DIV10,W
            MOVWF   TEMP20
            MOVF    DIV11,W
            MOVWF   TEMP21
            MOVF    DIV12,W
            MOVWF   TEMP22
            MOVF    DIV13,W
            MOVWF   TEMP23
            RETURN
;
TIM3        MOVF    TEMP20,W
            MOVWF   DIV1
            MOVF    TEMP21,W
            MOVWF   DIV2
            MOVF    TEMP22,W
            MOVWF   DIV3
            MOVF    TEMP23,W
            MOVWF   DIV4
;
            MOVF    DIV10,W
            MOVWF   DIV5
            MOVWF   TEMP30
            MOVF    DIV11,W
            MOVWF   DIV6
            MOVWF   TEMP31
            MOVF    DIV12,W
            MOVWF   DIV7
            MOVWF   TEMP32
            MOVF    DIV13,W
            MOVWF   DIV8
            MOVWF   TEMP33
;
            CALL    SUB_6
            CALL    SUB_7
            RETURN
;
TIM4        MOVF    DIV5,W
            MOVWF   DIV1
            MOVF    DIV6,W
            MOVWF   DIV2
            MOVF    DIV7,W
            MOVWF   DIV3
            MOVF    DIV8,W
            MOVWF   DIV4
;
DIST        MOVLW   0x00            ; 0.1 = 0.2 / 2
            CALL    DATA_RD
            MOVWF   DIV13
            MOVLW   0x01
            CALL    DATA_RD
            MOVWF   DIV14
            MOVLW   0x02
            CALL    DATA_RD
            MOVWF   DIV15
            MOVLW   0x03
            CALL    DATA_RD
            MOVWF   DIV16
            CALL    MULT
            RETURN
;
ARCSIN      CLRF    DIR
            CLRF    DIV1
            MOVLW   0xFB            ; 75.75 degrees
            MOVWF   DIV2
            MOVLW   0x14
            MOVWF   DIV3
            MOVLW   0xBE
            MOVWF   DIV4
            CALL    SUB_6
            CALL    SUB_7
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    ARCSIN_1       ; C=1 on positive result
            INCF    DIR,F
            CALL    SUB_6
            CALL    SUB_7          ; return to original value
            CLRF    DIV1
            MOVLW   0xD4           ; 56.25 degrees
            MOVWF   DIV2
            MOVLW   0xDB
            MOVWF   DIV3
            MOVLW   0x31
            MOVWF   DIV4
            CALL    SUB_6
            CALL    SUB_7
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    ARCSIN_1       ; C=1 on positive result
            INCF    DIR,F
            CALL    SUB_6
            CALL    SUB_7          ; return to original value
            CLRF    DIV1
            MOVLW   0x8E           ; 33.75 degrees
            MOVWF   DIV2
            MOVLW   0x39
            MOVWF   DIV3
            MOVLW   0xD9
            MOVWF   DIV4
            CALL    SUB_6
            CALL    SUB_7
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    ARCSIN_1       ; C=1 on positive result
            INCF    DIR,F
            CALL    SUB_6
            CALL    SUB_7          ; return to original value
            CLRF    DIV1
            MOVLW   0x31           ; 11.25 degrees
            MOVWF   DIV2
            MOVLW   0xF1
            MOVWF   DIV3
            MOVLW   0x70
            MOVWF   DIV4
            CALL    SUB_6
            CALL    SUB_7
            BTFSC   STATUS,C       ; C=0 on negative result
            GOTO    ARCSIN_1       ; C=1 on positive result
            INCF    DIR,F
            CALL    SUB_6
            CALL    SUB_7          ; return to original value
ARCSIN_1    BTFSC   BITS_BYTE,6
            GOTO    ARCSIN_2
            BTFSS   BITS_BYTE,5    ; +Y
            RETURN
            MOVF    DIR,W          ; -X
            SUBLW   0x10
            MOVWF   DIR
            MOVLW   0x10
            SUBWF   DIR,W
            BTFSC   STATUS,Z
            CLRF    DIR
            RETURN
ARCSIN_2    BTFSC   BITS_BYTE,5    ; -Y
            GOTO    ARCSIN_3
            MOVF    DIR,W
            SUBLW   0x08
            MOVWF   DIR
            RETURN
ARCSIN_3    MOVLW   0x08           ; -X
            ADDWF   DIR,F
            RETURN
;
SUB_6       MOVLW   0x30
SUB_64      MOVWF   FSR
            MOVLW   0x04            ; FF - divisor + carry
            MOVWF   CNT3
SUB_60      COMF    INDF,F
            INCF    FSR, F
            DECFSZ  CNT3, F
            GOTO    SUB_60
;
            BSF     STATUS,C        ; set carry
            MOVLW   0x04
            MOVWF   CNT3
SUB_61      DECF    FSR, F
            MOVF    INDF, W
            BTFSS   STATUS,C
            GOTO    SUB_62
            ADDLW   0x01
            GOTO    SUB_63
SUB_62      ADDLW   0x00
SUB_63      MOVWF   INDF
            DECFSZ  CNT3, F
            GOTO    SUB_61
            RETURN
;
SUB_7       MOVLW   0x04            ; dividend + (FF - divisor + carry)
            MOVWF   CNT3
            MOVLW   0x34
            MOVWF   FSR
            BCF     STATUS,C        ; clr carry
SUB_70      DECF    FSR, F
            MOVF    INDF, W
            INCF    FSR, F
            INCF    FSR, F
            INCF    FSR, F
            INCF    FSR, F
            BTFSS   STATUS,C
            GOTO    SUB_71
            ADDLW   0x01
            GOTO    SUB_72
SUB_71      ADDLW   0x00
SUB_72      BCF     BITS_B,4
            BTFSC   STATUS,C
            BSF     BITS_B,4        ; save carry flag
            ADDWF   INDF,F
            BTFSC   BITS_B,4
            BSF     STATUS,C
            DECF    FSR ,F
            DECF    FSR ,F
            DECF    FSR ,F
            DECF    FSR ,F
            DECFSZ  CNT3, F
            GOTO    SUB_70
            RETURN
;
MULT        MOVLW   0x20
            MOVWF   CNT1
            CLRF    DIV5
            CLRF    DIV6
            CLRF    DIV7
            CLRF    DIV8
            CLRF    DIV9
            CLRF    DIV10
            CLRF    DIV11
            CLRF    DIV12
            BCF     STATUS,C     
MULT1       RRF     DIV13,F         ; multiplier
            RRF     DIV14,F
            RRF     DIV15,F
            RRF     DIV16,F
            BTFSC   STATUS,C
            CALL    SUB_7
            RRF     DIV5,F
            RRF     DIV6,F
            RRF     DIV7,F
            RRF     DIV8,F
            RRF     DIV9,F
            RRF     DIV10,F
            RRF     DIV11,F
            RRF     DIV12,F
            DECFSZ  CNT1,F
            GOTO    MULT1
            RETURN
;
DIV_BIN0    CLRF    DIV12
DIV_BIN00   CLRF    DIV13
            CLRF    DIV14
            CLRF    DIV5
            CLRF    DIV6
            CLRF    DIV7
            CLRF    DIV8
;
            MOVF    DIV1,W         ; check for zero
            IORWF   DIV2,W
            IORWF   DIV3,W
            IORWF   DIV4,W
            BTFSC   STATUS,Z
            RETURN
;
            MOVLW   0x0C            ; move 12 times
            MOVWF   CNT1
            GOTO    DIV_BIN6
DIV_BIN1    CALL    SUB_6
DIV_BIN2    CALL    SUB_7
            BTFSS   STATUS,C        ; C=1 on positive result
            GOTO    DIV_BIN3        ; C=0 on negative result
            INCF    DIV14,F
            GOTO    DIV_BIN2
DIV_BIN3    CALL    SUB_6           ; return divisor to original value
            CALL    SUB_7           ; return dividend to original value
            DECF    CNT1, F
            BTFSS   STATUS, Z
            GOTO    DIV_BIN6
            MOVF    DIV8,W          ; remainer
            RETURN
DIV_BIN6    MOVLW   0x04
            MOVWF   CNT2
DIV_BIN4    MOVLW   0x0A            ; move 10 numbers to left, 4 places
            MOVWF   CNT3
            MOVLW   0x3E
            MOVWF   FSR
            CALL    DIV_5
            BTFSS   STATUS, Z
            GOTO    DIV_BIN4
            GOTO    DIV_BIN1
;
DIV_5       BCF     STATUS, C      ; shift to left 4 places
DIV_50      DECF    FSR, F
            RLF     INDF, F
            DECFSZ  CNT3, F
            GOTO    DIV_50
            DECF    CNT2, F
            RETURN
;
DEC_BIN     CLRF    DIV16           ; decimal to binary
            MOVF    DIV8,W
            BTFSC   STATUS,Z
            GOTO    DEC_BIN3
DEC_BIN2    INCF    DIV16,F
            MOVLW   0x99            ; -1
            ADDWF   DIV8,W
            CALL    DEC_ADD_0
            MOVWF   DIV8
            BTFSS   STATUS,Z
            GOTO    DEC_BIN2
DEC_BIN3    MOVF    DIV16,W
            RETURN
;
BIN_DEC_FR  CLRF    DIV17           ; binary fraction to decimal fraction
            CLRF    DIV18
            CLRF    DIV19
            CLRF    DIV20
            MOVLW   0x08
            MOVWF   CNT4
BIN_DEC_FR1 CLRF    DIV1
            CLRF    DIV13
            CLRF    DIV14
            CLRF    DIV15
            MOVLW   0x0A            ; x 10
            MOVWF   DIV16
            CALL    MULT
            MOVF    DIV9,W
            IORWF   DIV20,F
            DECFSZ  CNT4,F
            GOTO    BIN_DEC_FR2
            RETURN
BIN_DEC_FR2 MOVF    DIV10,W
            MOVWF   DIV2
            MOVF    DIV11,W
            MOVWF   DIV3
            MOVF    DIV12,W
            MOVWF   DIV4
;
            MOVLW   0x04            ; move 4 numbers to left, 4 places
            MOVWF   CNT2
BIN_DEC_FR3 BCF     STATUS,C
            RLF     DIV20,F
            RLF     DIV19,F
            RLF     DIV18,F
            RLF     DIV17,F
            DECFSZ  CNT2,F
            GOTO    BIN_DEC_FR3
            GOTO    BIN_DEC_FR1
;
DEC_BIN_FR  CLRF    DIV9         ; decimal fraction to binary fraction
            CLRF    DIV10
            CLRF    DIV11
            CLRF    DIV12
            MOVLW   0x20
            MOVWF   CNT1
DEC_BIN_FR1 CLRF    DIV1
            CLRF    DIV5
            MOVF    DIV6,W
            MOVWF   DIV2
            MOVF    DIV7,W
            MOVWF   DIV3
            MOVF    DIV8,W
            MOVWF   DIV4
            CALL    DIV_7        ; x 2
            MOVF    DIV5,W
            BTFSS   STATUS,Z
            BSF     DIV12,0
            DECFSZ  CNT1,F
            GOTO    DEC_BIN_FR2
            RETURN
DEC_BIN_FR2 BCF     STATUS,C
            RLF     DIV12,F
            RLF     DIV11,F
            RLF     DIV10,F
            RLF     DIV9,F
            GOTO    DEC_BIN_FR1
;
DIV_7       MOVLW   0x04         ; dividend + (99 - divisor + carry)
            MOVWF   CNT3
            MOVLW   0x34
            MOVWF   FSR
            BCF     STATUS,C     ; clr carry
DIV_70      DECF    FSR, F
            MOVF    INDF, W
            INCF    FSR, F
            INCF    FSR, F
            INCF    FSR, F
            INCF    FSR, F
            CALL    DEC_ADD_C
            ADDWF   INDF, W
            CALL    DEC_ADD_0
            MOVWF   INDF
            DECF    FSR ,F
            DECF    FSR ,F
            DECF    FSR ,F
            DECF    FSR ,F
            DECFSZ  CNT3, F
            GOTO    DIV_70
;
            MOVF    DIV5,W       ; check if result zero
            IORWF   DIV6,W
            IORWF   DIV7,W
            IORWF   DIV8,W
            RETURN
;
HEX_DEC     CLRF    DIV9         ; hex to decimal
HEX_DEC_0   CLRF    DIV7
            CLRF    DIV8
HEX_DEC_1   CLRF    DIV2
            CLRF    DIV1
            CLRF    DIV3
            CLRF    DIV5
            CLRF    DIV6
            BCF     STATUS,C
            BCF     STATUS,DC
            MOVF    DIV10,W
            ANDLW   0x0F
            BTFSC   STATUS,Z
            GOTO    HEX_DEC2
            CALL    DEC_ADD_0
            MOVWF   DIV4
            CALL    DIV_7
;
HEX_DEC2    SWAPF   DIV10,W
            ANDLW   0x0F
            BTFSC   STATUS,Z
            GOTO    HEX_DEC3
            MOVWF   CNT1
            MOVLW   0x16
            MOVWF   DIV4
            CALL    ADD_NUMS
;
HEX_DEC3    MOVF    DIV9,W
            ANDLW   0x0F
            BTFSC   STATUS,Z
            GOTO    HEX_DEC4
            MOVWF   CNT1
            MOVLW   0x02
            MOVWF   DIV3
            MOVLW   0x56
            MOVWF   DIV4
            CALL    ADD_NUMS
;
HEX_DEC4    SWAPF   DIV9,W
            ANDLW   0x0F
            BTFSC   STATUS,Z
            RETURN
            MOVWF   CNT1
            MOVLW   0x40
            MOVWF   DIV3
            MOVLW   0x96
            MOVWF   DIV4
            CALL    ADD_NUMS
            RETURN
;
ADD_NUMS    CALL    DIV_7
            DECFSZ  CNT1,F
            GOTO    ADD_NUMS    ; add numbers
            RETURN
;
DEC_ADD_C   BTFSS  STATUS,C
            GOTO   DEC_ADD_C1
            ADDLW  0x01            ; add carry
            BCF    STATUS,C
            RETURN
DEC_ADD_C1  ADDLW  0x00            ; no carry
            RETURN
;
DEC_ADD_0   MOVWF  DEC_S           ; decimal adjust
            BCF    BITS_B,4        ; save carry flag
            BTFSC  STATUS, C
            BSF    BITS_B,4
            BTFSC  STATUS, DC
            GOTO   ADD06
            ANDLW  0x0F
            SUBLW  0x09
            BTFSC  STATUS, C
            GOTO   DEC_ADD_1     ; if < 10
ADD06       MOVLW  0x06
            ADDWF  DEC_S, F
            BTFSC  STATUS,C
            BSF    BITS_B,4
DEC_ADD_1   BTFSC  BITS_B,4
            GOTO   ADD60
            SWAPF  DEC_S,W
            ANDLW  0x0F
            SUBLW  0x09
            BTFSC  STATUS, C
            GOTO   DEC_ADD_11     ; if < 10
ADD60       MOVLW  0x60
            ADDWF  DEC_S, F
            BTFSC  STATUS,C
            BSF    BITS_B,4
DEC_ADD_11  MOVF   DEC_S, W
            BCF    STATUS,C
            BTFSC  BITS_B,4
            BSF    STATUS,C
            RETURN
;
  end
